#include "poly.h"
#include "raster.h"
#include <math.h>

#ifdef ANSI
void ReadFormatData(char* DataFileName, char* ColorFileName, int datatype,
		    int nverts, int npolys, Poly_vert* vlist, Poly* plist)
#else
     void ReadFormatData(DataFileName, ColorFileName, datatype, nverts, npolys,
			 vlist, plist)
     char* DataFileName, ColorFileName;
     int datatype;
     int* nverts;
     int* npolys;
     Poly_vert* vlist;
     Poly* plist;
     
#endif
{
  double ** Vertices;
  int ** Polygons;
  int i;
  
  if (datatype == BINARY)
    Read_binary_polygon_data(DataFileName, nverts, Vertices, npolys, Polygons);
  else
    Read_ascii_polygon_data(DataFileName, nverts, Vertices, npolys, Polygons);
 
  vlist=(Poly_vert*)malloc(*nverts*sizeof(Poly_vert));
  for(i=0; i<(*nverts); i++) 
    {
      vlist[i].x = Vertices[i][0];
      vlist[i].y = Vertices[i][1];
      vlist[i].z = Vertices[i][2];
    }

  plist =(Poly*)malloc(*npolys*sizeof(Poly));
  if (!plist)
    myerror(3);
  for (i=0; i<(*npolys); i++)
    { 
      plist[i].vert[0]=Polygons[i][0];
      plist[i].vert[1]=Polygons[i][1];
      plist[i].vert[2]=Polygons[i][2];
    }
  for (i=0; i<(*nverts); i++)
    free(Vertices[i]);
  free(Vertices);
  for (i=0; i<(*npolys); i++)
    free(Polygons[i]);
  free(Polygons);
  
  if (datatype == BINARY)
    Read_binary_color(ColorFileName, nverts, Vertices);
  else
    Read_ascii_color(ColorFileName, nverts, Vertices);
  for(i=0; i<(*nverts); i++)
    {
      vlist[i].r = Vertices[i][0];
      vlist[i].g = Vertices[i][1];
      vlist[i].b = Vertices[i][2];
    }
  for (i=0; i<(*nverts); i++)
    free(Vertices[i]);
  free(Vertices);
}


#ifdef ANSI
void scan_convert(int nverts, Poly_vert* vlist, int npolys, Poly* plist, 
		  Poly_box box, Window win)
#else
     void scan_convert(nverts, vlist, npolys, plist, box, win)
     int nverts, npolys;
     Poly_vert* vlist;
     Poly* plist;
     Poly_box box;
     Window win;

#endif
{
  int i;
  
  for (i= 0; i< npolys; i++)
    {
      /*** Enable fields to interpolate ***/
      plist[i].mask = POLY_MASK(sx) | POLY_MASK(sy) | POLY_MASK(sz) | POLY_MASK(sw) |
	POLY_MASK(r) | POLY_MASK(g) | POLY_MASK(b);
      
      if (poly_clip_to_box(&plist[i], &box) == POLY_CLIP_OUT)	/* clip polygon */
	return;			  
  
      poly_scan(&plist[i], &win, pixelproc);	/* scan convert! */
    } 
}

#ifdef ANSI
void pixelproc(int x, int y, Poly_vert* pt)
#else
     void pixelproc(x, y, pt)
     int x, y;
     Poly_vert * pt;
#endif
{
  if (pt->sz > zbuffer[x][y].z)
    {
      zbuffer[x][y].r = pt->r;
      zbuffer[x][y].g = pt->g;
      zbuffer[x][y].b = pt->b;
      zbuffer[x][y].z = pt->z;
    }
}

#ifdef ANSI
void initialize_zbuffer(Z_point** zbuffer, double back_red, double back_green,
			double back_blue, int xmax, int ymax)
#else
     void initialize_zbuffer(zbuffer, back_red, back_green, back_blue, xmax, ymax)
     Z_point** zbuffer;
     double back_green, back_red, back_blue;
     int xmax, ymax;
#endif
{
  int i,j;
  for (i=0; i< xmax; i++)
    for(j=0; j< ymax; j++)
      {
	zbuffer[i][j].r = back_red;
	zbuffer[i][j].g = back_green;
	zbuffer[i][j].b = back_blue;
	zbuffer[i][j].z = -1 *HUGE;
      }
}



